#!/usr/bin/env python
# coding: utf-8


import pandas as pd # To read data files
import numpy as np # Process matrices
from matplotlib import pyplot as plt # Plot figures
from sklearn.decomposition import PCA # Perform PCA
from sklearn.cluster import KMeans # Perform K-Means .layers 
# Following packages are used to build the autoencoder
import tensorflow as tf
from tensorflow import keras
from tensorflow.keras.layers import Dense
from tensorflow.keras import models,Sequential

# Extract Long and Acute covid values 
X_acute_cyt=pd.read_csv(‘Acute_cytokine.csv’).iloc[:,:].values
X_long_cyt= pd.read_csv(‘Long_cytokine.csv’).iloc[:,:].values
X_acute_pro= pd.read_csv(‘Acute_proteins.csv’).iloc[:,:].values 
X_long_pro= pd.read_csv(‘Long_proteins.csv’).iloc[:,:].values
X_acute_meta= pd.read_csv(‘Acute_metabolities.csv’).iloc[:,:].values
X_long_meta= pd.read_csv(‘Long_metabolites.csv’).iloc[:,:].values
X_acute = np.hstack((X_acute_cyt,X_acute_pro,X_acute_meta))
X_long = np.hstack((X_long_cyt,X_long_pro,X_long_meta))
patient_labels = pd.read_csv(‘Patient labels.csv).iloc[:,:].values
X=X_long-X_acute
scaler=StandardScaler()
X=scaler.fit_transform(X) # Normalize the data
print(X.shape)

# Build AutoEncoder Architecture
model=Sequential()
model.add(Dense(100,input_shape=(782,),activation='sigmoid'))
model.add(Dense(70,activation='sigmoid'))
model.add(Dense(50,activation='sigmoid'))
model.add(Dense(30,activation='sigmoid'))
model.add(Dense(50,activation='sigmoid'))
model.add(Dense(70,activation='sigmoid'))
model.add(Dense(100,activation='sigmoid'))
model.add(Dense(782))
model.summary()
callback=tf.keras.callbacks.EarlyStopping(monitor ='loss', min_delta= 1E-7, patience= 1000, restore_best_weights= True) #Earlystopping
model.compile(loss='mse',optimizer='adam') # Compile model with MSE loss
model.fit(X,X,epochs=1000,verbose=1,callbacks =[callback]) # Fit model for 1000 epochs
Encoder=keras.Model(model.inputs,model.layers[3].output) #Extract the bottleneck layer to give lower dimensional features
Encoder.summary() # Print summary of model structure
X_low=Encoder.predict(X) # Generate lower dimensional features

#Create label vector and color vector for plotting
label_dict_comb={'recovered':'green','mild':'blue','severe':'red'}
class_dict_comb={'recovered':0,'mild':1,'severe':2}
cvec=[label_dict_comb[label] for label in patient_labels]
org_label=[class_dict_comb[label] for label in patient_labels]


kmeans_comb=KMeans(n_clusters=3).fit(X_low) # Perform Kmeans clustering with 3 clusters
cluster_new =kmeans_comb.labels_ #Store cluster labels for each sample

keras.models.save_model(model,'AutoEnc_Allfeatures_Delta_Sigmoid.hp5',save_format='h5') # Save AutoEncoder Model

#Save Clustered Data

dict={}
for i,key in enumerate(df.keys()[1:]):
    dict[key]=X[:,i]
dict['Cluster Label']=kmeans_comb.labels_
pd.DataFrame(dict).to_csv(‘Cluster labels.csv’)

# Plotting clustering results
s=np.ones(X.shape[0])*200
mpl.rcParams['figure.dpi']=1200
mpl.rcParams.update({'font.size':22})
colors=ListedColormap(['#69B0F8','#FEE0D2','#F2757B'])
colors1=ListedColormap(['magenta','yellow','black'])
fig,ax=plt.subplots(1,2,figsize=(20,20))
scatter=ax[0].scatter(X_low[:,0],X_low[:,1],s=s,c=org_label,cmap=colors,edgecolors='black')
ax[0].set_xlabel('Encoded Dim 1')
ax[0].set_ylabel('Encoded Dim 2')
ax[0].legend(handles=scatter.legend_elements()[0], labels=["Recovered",'Mild','Severe'],fontsize=35,markerscale=5)
scatter1=ax[1].scatter(X_low[:,0],X_low[:,1],s=s,c=cluster_new,cmap=colors1,edgecolors='black')
ax[1].set_xlabel('Encoded Dim 1')
ax[1].set_ylabel('Encoded Dim 2')
ax[1].legend(handles=scatter1.legend_elements()[0], labels=["Cluster A",'Cluster B', "Cluster C"],fontsize=35,markerscale=5)

# Function to find features with deviations above 65%
def find_deviations(df_clustered,cutoff,name):  
    means =df_clustered.abs().mean(numeric_only=True)
    cluster_means =df_clustered.abs().groupby(['ClusterLabels']).mean(numeric_only=True)
    deviation=cluster_means.sub(means[:-1]).div(means[:-1])
    features=deviation.copy()
    features[features.abs()>=cutoff]=1
    features[features.abs()<cutoff]=0        
    pd.concat([features,deviation.multiply(100)]).transpose().to_csv('Impute_Test_781/Features/Random_Impute_'+name+'.csv')
   
return features

#Find features in each cluster
print('Finding features for',name)
features = find_deviations(df_clustered,cutoff,name)
