library(readxl)
library(MetaboAnalystR)
library(ComplexHeatmap)
library(circlize)
library(dplyr)
library(ggplot2)

cytokines <- read_excel('Data.xlsx', sheet = 'Cytokines')
Proteins <- read_excel('Data.xlsx', sheet = 'Proteins')
Metabolites <- read_excel('Data.xlsx', sheet = 'Metabolites')

n_protein = 5 #number of the first protein column in your data 
n_metabolite = 5 #number of the first metabolite column in your data 
Data <- cbind(cytokines, 
              Proteins[n_protein:length(Proteins)], 
              Metabolites[n_metabolite:length(Metabolites)])

desc_cols = c(1:4) #number of the descriptive columns in the merged data
n_first_mol = 5 #number of the first molecule column in the merged data
Data <- cbind(Data[,desc_cols], 
              as.data.frame(sapply(Data[,n_first_mol:length(Data)],
                                   function (x) as.numeric(as.character(x)))))


sum(Data == 0, na.rm = TRUE)
Data <- replace(Data, Data == 0, NA)
sum(Data == 0, na.rm = TRUE) #Should return 0

Transposed_Data <- as.data.frame(t(Data))
## missing value vector:
p <-c()
for (i in 1:nrow(Transposed_Data)) {
 p[i] <- sum(is.na(Transposed_Data[i,]))/ncol(Transposed_Data)
   } 
#input the vector to the data
Transposed_Data <- Transposed_Data %>% 
  mutate(percent_of_missing_Values= p) %>% 
  select(percent_of_missing_Values, everything())
#filter the data based on the generated column (percent of #missing values)
missing_value_cut_off = 0.5
filtered_Transposed_Data <- Transposed_Data %>% 
  filter(percent_of_missing_Values < missing_value_cut_off)
##re-transpose the data:
cleaned_data <- as.data.frame(t(filtered_Transposed_Data))
cleaned_data <- cleaned_data[-1,]
row.names(cleaned_data) <- row.names(data)

# features containing more missing values than cut off 
eliminated_data <- Transposed_Data %>% 
    filter(percent_of_missing_Values > missing_value_cut_off)

## imputation of remaining missing values
sum(is.na(cleaned_data))
for (i in n_first_mol:ncol(cleaned_data)){
  for(j in 1:nrow(cleaned_data)){
    if (is.na(cleaned_data[j,i]) == "TRUE"){
      cleaned_data[j,i] = min(cleaned_data[,i], na.rm = TRUE)
    }
  }
}
##check NAs and zero values to be removed
sum(is.na(cleaned_data))
sum(cleaned_data[,n_first_mol:ncol(cleaned_data)]== 0, na.rm = TRUE)

########## Statistical Analysis
                                   
mSet<-InitDataObjects("conc", "stat", FALSE)
mSet<-Read.TextData(mSet, cleaned_data, "rowu", "disc")
mSet<-SanityCheckData(mSet)

mSet<-PreparePrenormData(mSet)
mSet<-Normalization(mSet, "NULL", "LogNorm", "MeanCenter", "NULL", ratio=FALSE, ratioNum=20)   

mSet<-PCA.Anal(mSet) #Perform PCA
mSet<-PlotPCAPairSummary(mSet, "pca_pair_0_", format = "png", dpi = 72, width=NA, 5) # Create PCA overview
mSet<-PlotPCAScree(mSet, "pca_scree_0_", "png", dpi = 72, width=NA, 5) # Create PCA scree plot
mSet<-PlotPCA2DScore(mSet, "pca_score2d_0_", format = "png", dpi=300, width=NA, 1, 2, 0.95, 0, 0) # Create a 2D PCA score plot

Fold_change_cut_off= 1.5 #set your cut off for fold change
P_Value_cut_off = 0.05 #set your cut off for adjusted p value
mSet<-Volcano.Anal(mSet, FALSE, FOld_change_cut_off, 0, F, P_Value_cut_off, TRUE, "fdr")  

mSet<-PlotVolcano(mSet, "volcano_0_", 1, 0, format ="png", dpi=300, width=NA)

top_mol = 100 #number of the top molecules to be shown in the heatmap
mSet<-PlotSubHeatMap(mSet, "heatmap_1_", "png", 300, width=NA,"norm", "row", "euclidean", "ward.D","bwm", 8,
                     "tanova", top_mol, "overview", F, T, T, F, T, T, T)

################### Correlation Analysis
                                   
clinical_variables<-read.csv("Data.csv")

n_o_clinical_variables = 20 
n_of_differentially_changed_molecules = 219
A = n_o_clinical_variables + n_of_differentially_changed_molecules
ncol_adjustment_var = 240
n__adjustment_var = 7
B= c(ncol_adjustment_var:(ncol_adjustment_var+n__adjustment_var))
                                   
#empty data frame for odds ratios
Coefficients<-data.frame(matrix(nrow = n_o_clinical_variables ,ncol = n_of_differentially_changed_molecules ))
row.names(Coefficients)<-colnames(clinical_variables [(n_of_differentially_changed_molecules+1):A])
colnames(Coefficients)<-colnames(clinical_variables [1:n_of_differentially_changed_molecules])

#empty data frame for p values
pvalue<-data.frame(matrix(nrow = n_o_clinical_variables, ncol = n_of_differentially_changed_molecules)) 
row.names(pvalue)<- row.names(Coefficients)
colnames(pvalue)<- colnames(Coefficients)

for(i in 1:n_of_differentially_changed_molecules){ 
    for(j in (n_of_differentially_changed_molecules+1):A){ 
        sym<-glm(unlist(clinical_variables [j])~unlist(clinical_variables [i])+
                 unlist(clinical_variables [B[1]])+ #list all your adjustment variables
                 unlist(clinical_variables [B[2]])+
                 unlist(clinical_variables [B[3]])+ 
                 unlist(clinical_variables [B[4]])+
                 unlist(clinical_variables [B[5]])+
                 unlist(clinical_variables [B[6]])+
                 unlist(clinical_variables [B[7]])+
                 unlist(clinical_variables [B[8]]),
                
                 family=binomial(),
                 data= clinical_variables)
        pvalue [j-n_of_differentially_changed_molecules,i]<-coef(summary(sym))[2,4]
        Coefficients [j-n_of_differentially_changed_molecules,i]<-exp(coef(summary(sym))[2,1])
        }  
} 

# transposing the matrix 
n.pval <- as.data.frame(t(pvalue)) 
#Make a column for the number of significant p values in each row for transposed table
v <- c()
for (i in 1:nrow(n.pval)){ 
  v[i]<- sum(n.pval[i,] < 0.05)
}
n.pval <- n.pval %>% 
  mutate(n.of.sig = v)
# subset the molecules with >= 3 significant p values
P_Value_cut_off = 3 #set your cut off for number of significant p vales
n.pval.filtered<- n.pval %>% 
  filter(n.of.sig >= P_Value_cut_off)
n.pval.filtered$n.of.sig <- NULL
#subset of odds ratio values of molecules with >= 3 significant p values
odds.filrtered <- Coefficients %>% 
  select(rownames(n.pval.filtered))

odds.filrtered <- as.data.frame(t(odds.filrtered))


my_color_mapping <- colorRamp2(c(0, 1, 10), c("#2E2EFF", "white", "#FF2E2E"))
Heatmap(  
   as.matrix(odds.filrtered),
  width = ncol(odds.filrtered)*unit(4.5, "mm"), 
  height = nrow(odds.filrtered)*unit(3, "mm"),
  cluster_rows = FALSE,
  cluster_columns = FALSE,
  show_row_dend = FALSE,
  show_column_dend = FALSE,
  clustering_method_rows = "ward.D2",
  heatmap_legend_param = list(
     title = "Odds Ratio",
     at = seq(0, 2, length.out = 3),
     labels = c( "10^-5", "1", "10^5")),
  cell_fun = function(j, i, x, y, w, h, fill) {
    if(n.pval.filtered [i, j] < 0.05) {
       grid.text("*", x , y-(0.3*h) ,gp=gpar(fontsize=18))
      }
                },
  col = my_color_mapping,
  column_names_gp = grid::gpar(fontfamily= "Arial", fontface= "bold", fontsize = 10),
  row_names_gp = grid::gpar(fontfamily= "Arial", fontface= "bold",      fontsize = 9)
)










